\\ Copyright 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Whether to print test progress and names.
if(option_verbose=='option_verbose, option_verbose=0);

\\ Whether to setrand(), default 1 = yes.
if(option_randomize=='option_randomize, option_randomize=1);


\\-----------------------------------------------------------------------------

\\ randomize by getwalltime() in gp 2.9 up, or C library time() in gp 2.7
test_have_getwalltime = iferr(getwalltime();1,e,0);
if(option_verbose, print("test_have_getwalltime = "test_have_getwalltime));
test_have_C_library_time=0;
{
  if(!test_have_getwalltime,
     if(default(secure),
        if(option_verbose, print("not trying install() in secure mode"));
        ,
        test_have_C_library_time
          = iferr(install(time, "lf", "test_C_library_time");1,e,0));
     if(option_verbose, print("test_have_C_library_time = "test_have_C_library_time)));
}
{
  if(option_randomize,
     my(seed=1);
     if(test_have_getwalltime,
        seed=getwalltime()%2^32;
        print("setrand() seed="seed" (from getwalltime())");
        ,
        test_have_C_library_time,
        seed=test_C_library_time();
        print("setrand() seed="seed" (from time())");
        ,
        print("setrand() seed="seed" (no getwalltime(), and no install() for C time())"));
     setrand(seed));
}

\\ Uncomment this to force random to particular seed.
\\ setrand(3553445209);


\\-----------------------------------------------------------------------------

check_count = 0;

\\ Check that got==want.
\\
\\ ENHANCE-ME: Would like to report caller's source file name and line
\\ number.  For now some identifying "name" parameter only.
\\ 
check_equal(got,want,name="") =
{
  check_count++;
  if(! (got == want),
     print("wrong at "check_count": "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

nearly_equal_epsilon = 1e-10;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) =
{
  my(d=abs(x*1.0 - y*1.0));
  if(type(d)=="t_MAT", d=vecsum(Vec(d)));
  if(type(d)=="t_VEC" || type(d)=="t_COL", d=vecsum(d));
  abs(d) <= epsilon;
}
check_nearly(got,want, name="", epsilon=nearly_equal_epsilon) =
{
  check_count++;
  if(! nearly_equal(got,want,epsilon),
     print("wrong "name);
     print("got  "got*1.0);
     print("want "want*1.0);
     print("epsilon "epsilon*1.0);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

\\ Return a vector of the first n coefficients of
\\ generating function g, starting from term x^0.
\\ g is normally a polynomial fraction (type t_RFRAC)
\\ but a plain polynomial is accepted too and returns
\\ its lowest n coefficients.
gf_terms(g,n) =
{
  if(g==0,return(vector(n)));
  my(x = variable(g), 
     zeros = min(n,valuation(g,x)), 
     v = Vec(g + O(x^n))); 
  if(zeros>=0, concat(vector(zeros,i,0), v), 
               v[-zeros+1 .. #v]);
}

check_equal(gf_terms(0, 3), [0,0,0]);
check_equal(gf_terms(1/(1-2*x), 3), [1,2,4]);
check_equal(gf_terms(x^2/(1-2*x), 3), [0,0,1]);
check_equal(gf_terms(x^3/(1-2*x), 3), [0,0,0]);
check_equal(gf_terms(x^4/(1-2*x), 3), [0,0,0]);
check_equal(gf_terms(x^10/(1-2*x), 3), [0,0,0]);
check_equal(gf_terms(1+2*x+5*x^2, 2), [1,2]);


\\-----------------------------------------------------------------------------
