\\ Copyright 2015, 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("geom.gp");
read("geom.gp");    \\ safe to multiple reads
read("geom.gp");
read("test-helpers.gp");

option_verbose = 0;


\\-----------------------------------------------------------------------------
\\ geom_dot_product()

check_equal(geom_dot_product(0,0), 0);
check_equal(geom_dot_product(1+2*I, 3+4*I), 1*3+2*4);


\\-----------------------------------------------------------------------------
\\ geom_quadrant_of_point()

check_equal(geom_quadrant_of_point(0), 0);
check_equal(geom_quadrant_of_point([0,0,0]), 0);
{
  my(b=1+I);
  for(i=0,7,
     check_equal(geom_quadrant_of_point(b^i), (i\2)+1,
                 Str("geom_quadrant_of_point() eighths around i="i)));
}


\\-----------------------------------------------------------------------------
\\ geom_arg_positive()

check_nearly(geom_arg_positive(1), 0, "geom_arg_positive() of 1");
check_nearly(geom_arg_positive(-1), Pi);
check_nearly(geom_arg_positive(-I), 3/2*Pi);
check_nearly(geom_arg_positive(1-I) > 0, 1);


\\-----------------------------------------------------------------------------
\\ geom_xydir_cmp()

check_equal(geom_xydir_cmp(0,0), 0);
check_equal(geom_xydir_cmp(1,0), 1);
check_equal(geom_xydir_cmp(0,1), -1);
{
  my(b=1+I);
  for(i=0,7,
     for(j=0,7,
        check_equal(geom_xydir_cmp(b^i,b^j), lex(i,j),
                    "geom_xydir_cmp() eighths around")));
}

check_equal(geom_xydir_cmp(0,0),  0);
check_equal(geom_xydir_cmp(I,-I), -1);   /* p1 smaller */
check_equal(geom_xydir_cmp(-I,I),  1);
check_equal(geom_xydir_cmp(-1,I),  1);   /* p1 bigger */
check_equal(geom_xydir_cmp(-1,-I), -1);
check_equal(geom_xydir_cmp(1,1+I), -1);
check_equal(geom_xydir_cmp(1,I), -1);
check_equal(geom_xydir_cmp(1,-1+I), -1);
check_equal(geom_xydir_cmp(1,1), 0);
check_equal(geom_xydir_cmp(-1,-1), 0);
check_equal(geom_xydir_cmp(3,-3), -1);  /* p1 smaller */
check_equal(geom_xydir_cmp(-1,1),  1);
check_equal(geom_xydir_cmp(-3, 22+4*I), 1);  /* p1 bigger */
check_equal(geom_xydir_cmp(8+6*I, 2+4*I), -1);
check_equal(cmp(arg(8+6*I), arg(2+4*I)), -1);

\\ for z1,z2 complex numbers
test_xydir_cmp_by_arg(z1,z2) =
{
  if(z1==0 || z2==0, lex(norm(z1),norm(z2)),
     lex(geom_arg_positive(z1),geom_arg_positive(z2)));
}
{
  for(x1=-4,4,
     for(y1=-2,2,
        my(z1=x1+y1*I);
        for(x2=-4,4,
           for(y2=-2,2,
              my(z2=x2+y2*I);
              my(got    =geom_xydir_cmp(z1,z2),
                 by_arg=test_xydir_cmp_by_arg(z1,z2));
              check_equal(got, by_arg)))));
}


\\-----------------------------------------------------------------------------
\\ geom_minrect_area_of_points_at_xydir()

check_equal(geom_minrect_area_of_points_at_xydir([0,2, 2+I], 1), 2);
check_equal(geom_minrect_area_of_points_at_xydir([0,2, 2+I], I), 2);
check_equal(geom_minrect_area_of_points_at_xydir([0,1, 1+I, I], 1+I), 2);
check_equal(geom_minrect_area_of_points_at_xydir([0,1, 1+I, I], 1),   1);


\\-----------------------------------------------------------------------------
\\ geom_minrect_side_of_polygon()
\\ geom_minrect_area_of_polygon()

check_equal(geom_minrect_side_of_polygon([0,1, 1+I, I]),   1);
check_equal(geom_minrect_area_of_polygon([0,1, 1+I, I]),   1);


\\-----------------------------------------------------------------------------
\\ geom_area_of_intformal_rows()

check_equal(geom_area_of_intformal_rows(0,1, 0,'y), 1/2);


\\-----------------------------------------------------------------------------
\\ geom_length_of_path()

check_equal(geom_length_of_path([0,1,1+I]), 2);


\\-----------------------------------------------------------------------------
\\ geom_point_rotate_xydir()
\\ geom_points_rotate_xydir()

check_equal(geom_point_rotate_xydir([1,0], I), [0,1]);
check_equal(geom_points_rotate_xydir([1,0], I), [[0,1], [0,0]]);


\\-----------------------------------------------------------------------------
\\ geom_sqrt_as_quad()

sqrt2 = quadgen(4*2);
check_equal(sqrt2^2, 2);

sqrt3 = quadgen(4*3);
check_equal(sqrt3^2, 3);

sqrt3i = quadgen(-12);  /* sqrt(3)*I */
check_equal(sqrt3i^2, -3);

sqrt5 = geom_sqrt_as_quad(5);
check_equal(sqrt5^2, 5);

w3 = -1/2 + 1/2*sqrt3i;
check_equal(w3^3, 1);
w6 = w3 + 1;
check_equal(w6^3, -1);
check_equal(w6^6, 1);

w6_at_60deg = quadgen(-3);     /* (1 + sqrt(-3))/2 */
check_equal(w6_at_60deg^3, -1);
check_equal(w6_at_60deg^6, 1);

w3_at_60deg = w6_at_60deg - 1;
check_equal(w3_at_60deg^3, 1);

check_nearly(w3, w3_at_60deg);
check_nearly(w6, w6_at_60deg);

sqrt21 = quadgen(4*21);
check_equal(sqrt21^2, 21);
check_equal(geom_sqrt_as_quad(21), sqrt21);
check_equal(geom_sqrt_as_quad(3/7), 1/7*sqrt21);

check_equal(vector(1000,n,n--; geom_sqrt_as_quad(n)^2 == n), \
            vector(1000,n,n--; 1));
check_equal(geom_sqrt_as_quad(1), 1);
check_equal(geom_sqrt_as_quad(2), sqrt2);
check_equal(geom_sqrt_as_quad(4), 2);
check_equal(geom_sqrt_as_quad(8), 2*sqrt2);
check_equal(geom_sqrt_as_quad(16), 4);
check_equal(geom_sqrt_as_quad(1/8), 1/(2*sqrt2));
check_equal(geom_sqrt_as_quad(1/16), 1/4);

\\ per addhelp() docs
check_equal(geom_sqrt_as_quad(75), 5*geom_sqrt_as_quad(3));
check_equal(geom_sqrt_as_quad(3), sqrt3);
check_equal(geom_sqrt_as_quad(3), quadgen(12));


\\-----------------------------------------------------------------------------
\\ geom_norm()

check_equal(geom_norm(1+I), norm(1+I));
check_equal(geom_norm(1+sqrt3i), norm(1+sqrt3i));
check_equal(geom_norm([3,4]), 25);

check_equal(geom_norm([geom_sqrt_as_quad(3), \
                       geom_sqrt_as_quad(5), \
                       geom_sqrt_as_quad(7)]), 3+5+7);


\\-----------------------------------------------------------------------------
\\ geom_INTERNAL_hypot()

check_equal(geom_INTERNAL_hypot(3,4), 5);
check_equal(geom_INTERNAL_hypot(12,5), 13);
check_equal(geom_INTERNAL_hypot(1,2,5), geom_sqrt_as_quad(1^2+2^2+5^2));


\\-----------------------------------------------------------------------------
\\ geom_xydir_half()

check_equal(geom_xydir_half([0,1]), [1,1]);
check_equal(geom_xydir_half([1,1]), [1+sqrt2,1]);

check_equal(geom_xydir_half([0,-1]), [1,-1]);
check_equal(geom_xydir_half([1,-1]), [1+sqrt2,-1]);

check_equal(geom_xydir_half([-4,3]), 3*[1,3]);
check_equal(geom_xydir_half([-100,1]), [1,100+geom_sqrt_as_quad(10001)]);
check_equal(geom_xydir_half([-5,0]), [0,5]);  \\ -X rotates up to Y axis

\\ (a+b*I)*(1+s*I) == a-s*b + (s*a+b)*I
\\ +90 [-y,x]
\\ half [-y+h,x]
\\ -45  [x-y+h, x+y-h]
test_xydir_half_by_vertical(p) =
{
  my(x = geom_point_x(p),
     y = geom_point_y(p),
     h = geom_INTERNAL_hypot(x,y));
  if(x<0, [y, h-x]*sign(y), [x+h,y]);
}
check_equal((1+2*I)^2, -3+4*I);
check_equal((1+3*I)^2, 2*(-4+3*I));
check_equal(test_xydir_half_by_vertical([3,4]), 4*[2,1],    "1st");
check_equal(test_xydir_half_by_vertical([3,-4]), 4*[2,-1],  "4th");
check_equal(test_xydir_half_by_vertical([-3,4]), 4*[1,2],   "2nd");
check_equal(test_xydir_half_by_vertical([-3,-4]), 4*[1,-2], "3rd");

\\ forvec(p=[[-4,4],[-4,4]],print(p));

\\ (2+I)^2 == 3 + 4*I
\\ (2-I)^2 == 3 - 4*I
\\ (1+3*I)^2 == -8 + 6*I
\\ (1-3*I)^2 == -8 - 6*I

\\ sqrt(-8 + 6*I)
\\ sqrt(-8 - 6*I)

\\ 2*Re(p^2)*Im(p^2) = Im(p)

\\ X = sqrt (h/2 + x/2);
\\ Y = y/2 / X;

\\ x>0
\\ X = h + x
\\ Y = y

\\ x<0
\\ r = y/2
\\ s = d/2 - x/2
\\ r = y
\\ s = d - x

\\ x=-3; y=-4; d=geom_INTERNAL_hypot(x,y)
\\ r = y/2/sqrt(d/2 - x/2)
\\ s = sqrt(d/2 - x/2)
\\ r = abs(y/2/s) *sign(y)
\\ (r + I*s)^2
\\ (-3-4*I)*4*4

\\ x=-3; y=4; d=geom_INTERNAL_hypot(x,y)
\\ r = sqrt(d/2 + x/2)
\\ s = (y/2/s)
\\ (r + I*s)^2


\\-----------------------------------------------------------------------------
\\ geom_abs()

check_equal(geom_abs(0), 0);
check_equal(geom_abs(1), 1);
check_equal(geom_abs(I), 1);
check_equal(geom_abs([3,4]), 5);
check_equal(geom_abs([1,1,1]), sqrt3);
check_equal(geom_abs([5,0,0]), 5);
check_equal(geom_abs([0,5,0]), 5);
check_equal(geom_abs([0,0,5]), 5);

check_equal(geom_abs([geom_sqrt_as_quad(3), \
                      geom_sqrt_as_quad(5), \
                      geom_sqrt_as_quad(7)]), geom_sqrt_as_quad(3+5+7));

\\-----------------------------------------------------------------------------
\\ geom_point_x()
\\ geom_point_y()
\\ geom_point_z()

check_equal(geom_point_x(   w3), -1/2, "x w3");
check_equal(geom_point_x('r*w3), -'r/2, "x r*w3");
check_equal(geom_point_x(('a+'b)*w3), -('a+'b)/2);


check_equal(geom_point_y(   w3),    1/2*sqrt3, "y w3");
check_equal(geom_point_y('r*w3), 'r*1/2*sqrt3, "y r*w3");
check_equal(geom_point_y(('a+'b)*w3), ('a+'b)*1/2*sqrt3, "y r*w3");

\\ RFRAC
{
  my(g='x/(1+I*'x),
     xg=geom_point_x(g),
     yg=geom_point_y(g));
  check_equal(gf_terms(g,6),  [0, 1, -I, -1, I, 1]);
  check_equal(gf_terms(xg,6), [0, 1,  0, -1, 0, 1]);
  check_equal(gf_terms(yg,6), [0, 0, -1,  0, 1, 0], "yg I terms");
  check_equal(xg, 'x/(1+'x^2),     "xg I");
  check_equal(yg, -'x^2/(1+'x^2));
  check_equal(geom_point_z(g), 0);
  check_equal(xg+I*yg,g);
}
{
  my(g='x/(1+w3*'x),
     xg=geom_point_x(g),
     yg=geom_point_y(g));
  check_equal(xg, (2*'x - 'x^2)/(2 - 2*'x + 2*'x^2), "xg w3");
  check_equal(yg,  -sqrt3*'x^2 /(2 - 2*'x + 2*'x^2), "yg w3");
  check_equal(geom_point_z(g), 0);

  my(v=gf_terms(g,8));
  check_equal(v, [0, 1, 1/2-1/2*sqrt3i, -1/2-1/2*sqrt3i,
                  -1, -1/2+1/2*sqrt3i, 1/2+1/2*sqrt3i, 1]);
  check_equal(gf_terms(xg,8), apply(geom_point_x,v), "xg w3 terms");
  check_equal(gf_terms(yg,8), apply(geom_point_y,v));

  check_equal(g,xg + sqrt3i*(simplify(yg/sqrt3)));

  \\ print(v);
  \\ print(apply(geom_point_y,v));
  \\ print(gf_terms(yg,8));
  \\ print((v[3]).disc);
  \\ print((geom_point_y(v[3])).disc);
  \\ print((gf_terms(yg,8)[3]).disc);
}

check_equal(geom_point_x('x/(1+'x)*w3), -'x/(1+'x)/2);
check_equal(geom_point_y('x/(1+'x)*w3), 'x/(1+'x)*1/2*sqrt3);

check_equal(geom_point_z(w3), 0);
check_equal(geom_point_z(('a+'b)*w3), 0);

check_equal(geom_point_x(w6), 1/2);
check_equal(geom_point_y(w6), 1/2*sqrt3);
check_equal(geom_point_z(w6), 0);

check_equal(geom_point_x(w3_at_60deg), -1/2, "w3_at_60deg x");
check_equal(geom_point_y(w3_at_60deg), 1/2*sqrt3, "w3_at_60deg y");

check_equal(geom_point_x(w6_at_60deg), 1/2);
check_equal(geom_point_y(w6_at_60deg), 1/2*sqrt3, "w6_at_60deg y");

check_equal(geom_point_x([10,20,30]), 10);   \\ vector
check_equal(geom_point_y([10,20,30]), 20, "y of vec3");
check_equal(geom_point_z([10,20,30]), 30, "z of vec3");
check_equal(geom_point_x([10,20]), 10);
check_equal(geom_point_y([10,20]), 20);
check_equal(geom_point_z([10,20]), 0);

check_equal(geom_point_x([10,20,30]~), 10);  \\ column
check_equal(geom_point_y([10,20,30]~), 20);
check_equal(geom_point_z([10,20,30]~), 30);
check_equal(geom_point_x([10,20]~), 10);
check_equal(geom_point_y([10,20]~), 20);
check_equal(geom_point_z([10,20]~), 0);

check_equal(geom_point_x([10;20;30]), 10);  \\ matrix
check_equal(geom_point_y([10;20;30]), 20);
check_equal(geom_point_z([10;20;30]), 30);
check_equal(geom_point_x([10;20]), 10);
check_equal(geom_point_y([10;20]), 20);
check_equal(geom_point_z([10;20]), 0);


\\-----------------------------------------------------------------------------
\\ geom_length_of_segment()

check_equal(geom_length_of_segment(0,1), 1);
check_equal(geom_length_of_segment(0,I), 1);
check_equal(geom_length_of_segment(0,1+I), geom_sqrt_as_quad(2));
check_equal(geom_length_of_segment(0,[1,1,1]), geom_sqrt_as_quad(3));
check_equal(geom_length_of_segment(0,[1,2,3]), geom_sqrt_as_quad(1+4+9));


\\-----------------------------------------------------------------------------
\\ geom_area_of_circle()

check_equal(geom_area_of_circle(0), 0);
check_nearly(geom_area_of_circle(1), Pi);
check_nearly(geom_area_of_circle(2), 4*Pi);


\\-----------------------------------------------------------------------------
\\ geom_area_of_ellipse()

check_equal(geom_area_of_ellipse(0,0), 0);
check_equal(geom_area_of_ellipse(1,0), 0);
check_equal(geom_area_of_ellipse(0,1), 0);
check_nearly(geom_area_of_ellipse(1,1), Pi);


\\-----------------------------------------------------------------------------
\\ geom_area_of_rectangle5()

check_equal(geom_area_of_rectangle5(1,0, 1,I,0), 1);
check_equal(geom_area_of_rectangle5(0,1, 1,I,0), 1);
check_equal(geom_area_of_rectangle5(0,1, 0,I,1), -1);
check_equal(geom_area_of_rectangle5(-2,5, 1, I, 0), 1);
check_equal(geom_area_of_rectangle5(-2,5, 1-5*I, 6+I, 0+7*I), 1);
check_equal(geom_area_of_rectangle5(-2,5, 1,sqrt3i,0), sqrt3);
check_equal(geom_area_of_rectangle5(-2,5, 1, w6_at_60deg,0), sqrt3/2);

check_equal(geom_area_of_rectangle5(-100,100, 1,I,0), 1);


\\-----------------------------------------------------------------------------
\\ geom_area_of_triangle()

check_equal(geom_area_of_triangle(1,I), 1/2, "triangle area 1,I");
check_equal(geom_area_of_triangle(2,I), 1, "triangle area 2,I");
check_equal(geom_area_of_triangle(4,3*I), 6, "triangle area 4,3I");
check_equal(geom_area_of_triangle([4,0],[0,3]), 6, "triangle area 4,3I");
check_equal(geom_area_of_triangle([sqrt2,0],[0,sqrt2]), 1);
check_equal(geom_area_of_triangle([sqrt2,0],[sqrt2,sqrt2],[0,sqrt2]), 1);

check_equal(geom_area_of_triangle(w6,w3), 1/4*sqrt3, "triangle area w3,w6");
check_equal(geom_area_of_triangle(w6_at_60deg,w3_at_60deg), 1/4*sqrt3);

check_equal(geom_area_of_triangle(1+w6, 1+w3, 1), 1/4*sqrt3);
check_equal(geom_area_of_triangle(1+w6_at_60deg, 1+w3_at_60deg, 1), 1/4*sqrt3);

check_equal(geom_area_of_triangle(0, w6, w3), 1/4*sqrt3);
check_equal(geom_area_of_triangle(0, w3, w6), -1/4*sqrt3, "clockwise negative");

\\ example in the docs
check_equal(geom_area_of_triangle(['b,0],[0,'h]), 1/2*'b*'h);

test_cross_product(p1,p2) =
{
  my(x1=geom_point_x(p1),
     y1=geom_point_y(p1),
     z1=geom_point_z(p1),
     x2=geom_point_x(p2),
     y2=geom_point_y(p2),
     z2=geom_point_z(p2));
  [ y1*z2 - z1*y2,
    z1*x2 - x1*z2,
    x1*y2 - y1*x2 ];
}

test_area_of_triangle_by_cross_product(p1,p2,p3=0) =
{
  p1 -= p3;
  p2 -= p3;
  test_cross_product(p1,p2) / 2;
}

check_equal(geom_area_of_triangle(['a,'b],['c,'d],['e,'f]), \
            test_area_of_triangle_by_cross_product(['a,'b],['c,'d],['e,'f])[3]);

check_equal(geom_area_of_triangle(['a,'b],['c,'d],['e,'f]), \
            -test_area_of_triangle_by_cross_product(['a,0,'b],['c,0,'d],['e,0,'f])[2]);

test_cross_product_by_matdet(p1,p2) =
{
  my(x1=geom_point_x(p1),
     y1=geom_point_y(p1),
     z1=geom_point_z(p1),
     x2=geom_point_x(p2),
     y2=geom_point_y(p2),
     z2=geom_point_z(p2));
  [ matdet([y1,z1;y2,z2]),
   -matdet([x1,z1;x2,z2]),
    matdet([x1,y1;x2,y2]) ];
}
check_equal(test_cross_product([x1,y1,z1],[x2,y2,z2]), \
            test_cross_product_by_matdet([x1,y1,z1],[x2,y2,z2]));

\\ 3D needs a geom_abs() on the cross product vector, unless all in XY, XZ,
\\ YZ etc plane.  What sense for anti-clockwise in that case?
\\ check_equal(geom_area_of_triangle([1,0,0],[0,0,1]), 1/2);

\\ No, can't give generating function to anything which multiplies as the
\\ product is not term-wise.
\\ {
\\   my(g=1/(1-2*x),
\\      ag=geom_area_of_triangle([g,0],[0,g]));
\\   check_equal(gf_terms(g,5),  [1,2,4,8,16]);
\\   check_equal(gf_terms(ag,12), [1/2, 4/2, 16/2, 64/2, 256/2]);
\\   check_equal(ag, (1/2)/(1-4*x));
\\ }

\\ geom_area_of_triangle3(z1,z2,z3) = \
\\   -I/2 * matdet([1,1,1; z1,z2,z3; conj(z1),conj(z2),conj(z3)]);
\\ geom_area_of_triangle(z1,z2) = geom_area_of_triangle3(z1,z2,0);
\\ geom_area_of_triangle(z1,z2) = I/2 * (conj(z1)*z2 - z1*conj(z2));
\\ geom_area_of_triangle(z1,z2) = I/2 * (conj(z1)*z2 - conj(conj(z1)*z2));
\\ geom_area_of_triangle(z1,z2) = imag(z1*conj(z2))/2;


\\-----------------------------------------------------------------------------
\\ geom_area_of_polygon()

check_equal(geom_area_of_polygon([]), 0, "geom_area_of_polygon() empty");
check_equal(geom_area_of_polygon([0]), 0);
check_equal(geom_area_of_polygon([0,0]), 0);
check_equal(geom_area_of_polygon([0,1]), 0);
check_equal(geom_area_of_polygon([1]), 0);
check_equal(geom_area_of_polygon([1,1]), 0);
check_equal(geom_area_of_polygon([1,2]), 0);
check_equal(geom_area_of_polygon([1,2,3]), 0);   \\ straight line
check_equal(geom_area_of_polygon([0,1,1+I]), 1/2);
check_equal(geom_area_of_polygon([0,1+I,1]), -1/2);
check_equal(geom_area_of_polygon([2,3,3+I,2+I]), 1);  \\ away from origin

check_equal(geom_area_of_polygon([ [0,0], [1,0], [1,1], [0,1] ]), 1);


\\-----------------------------------------------------------------------------
\\ geom_centroid_of_triangle()

check_equal(geom_centroid_of_triangle(0, 2, 1+I), 1+1/3*I, \
            "geom_centroid_of_triangle()");
check_equal(geom_centroid_of_triangle(2, 1+I), 1+1/3*I, \
            "geom_centroid_of_triangle()");
check_equal(geom_centroid_of_triangle([0,0], [2,0], [1,1]), [1,1/3], \
            "geom_centroid_of_triangle()");
check_equal(geom_centroid_of_triangle([2,0], [1,1]), [1,1/3], \
            "geom_centroid_of_triangle()");


\\-----------------------------------------------------------------------------
\\ geom_centroid_of_points()

check_equal(geom_centroid_of_points([1,-1]), 0);
check_equal(geom_centroid_of_points([[1,0],[0,1]]), [1/2,1/2]);


\\-----------------------------------------------------------------------------
\\ geom_centroid_of_polygon()

check_equal(geom_centroid_of_polygon([0,1,1+I,I]), 1/2+1/2*I, \
            "geom_centroid_of_polygon()");


\\-----------------------------------------------------------------------------
\\ geom_sides_of_polygon()

check_equal(geom_sides_of_polygon([]), [], "geom_sides_of_polygon() empty");
check_equal(geom_sides_of_polygon([0]), [0]);
check_equal(geom_sides_of_polygon([0,1]), [1, -1]);
check_equal(geom_sides_of_polygon([0,1,1+I,I]), [1,I,-1,-I]);


\\-----------------------------------------------------------------------------
\\ geom_length_of_polygon()

check_equal(geom_length_of_polygon([0]), 0);
check_equal(geom_length_of_polygon([0,1,1+I,I]), 4, "boundary of polygon square");
check_equal(geom_length_of_polygon([1,2,3]), 4, "boundary of polygon 3");
check_equal(geom_length_of_polygon([0,1]), 2, "boundary of polygon");
check_equal(geom_length_of_polygon([0,1,1+I]), 2+geom_sqrt_as_quad(2));


\\-----------------------------------------------------------------------------

\\ All variables in geom.gp not left as globals.
{
  my(test_symbols=readstr("test-symbols.txt"));
  check_equal(#test_symbols>0,1);
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     check_equal(eval(test_symbol),
                 eval(Str("'"test_symbol)),
                 Str("check no leftover global variable: "test_symbol)));
}

\\-----------------------------------------------------------------------------

print("test-geom.gp end, ",check_count," tests");
quit(0);
